@@ -78,11 +78,10 @@ gth_metadata_provider_exiv2_can_write (GthMetadataProvider  *self,
 static void
 gth_metadata_provider_exiv2_read (GthMetadataProvider *self,
 				  GthFileData         *file_data,
-				  const char          *attributes)
+				  const char          *attributes,
+				  GCancellable        *cancellable)
 {
-	char        *uri;
-	char        *uri_wo_ext;
-	char        *sidecar_uri;
+	GFile       *sidecar;
 	GthFileData *sidecar_file_data;
 
 	if (! g_content_type_is_a (gth_file_data_get_mime_type (file_data), "image/*"))
@@ -91,24 +90,20 @@ gth_metadata_provider_exiv2_read (GthMetadataProvider *self,
 	/* this function is executed in a secondary thread, so calling
 	 * slow sync functions is not a problem. */
 
-	exiv2_read_metadata_from_file (file_data->file, file_data->info, NULL);
+	exiv2_read_metadata_from_file (file_data->file, file_data->info, cancellable, NULL);
 
 	/* sidecar data */
 
-	uri = g_file_get_uri (file_data->file);
-	uri_wo_ext = _g_uri_remove_extension (uri);
-	sidecar_uri = g_strconcat (uri_wo_ext, ".xmp", NULL);
-	sidecar_file_data = gth_file_data_new_for_uri (sidecar_uri, NULL);
-	if (g_file_query_exists (sidecar_file_data->file, NULL)) {
+	sidecar = exiv2_get_sidecar (file_data->file);
+	sidecar_file_data = gth_file_data_new (sidecar, NULL);
+	if (g_file_query_exists (sidecar_file_data->file, cancellable)) {
 		gth_file_data_update_info (sidecar_file_data, "time::*");
-		if (g_file_query_exists (sidecar_file_data->file, NULL))
+		if (g_file_query_exists (sidecar_file_data->file, cancellable))
 			exiv2_read_sidecar (sidecar_file_data->file, file_data->info);
 	}
 
 	g_object_unref (sidecar_file_data);
-	g_free (sidecar_uri);
-	g_free (uri_wo_ext);
-	g_free (uri);
+	g_object_unref (sidecar);
 }
 
 
@@ -116,12 +111,14 @@ static void
 gth_metadata_provider_exiv2_write (GthMetadataProvider   *self,
 				   GthMetadataWriteFlags  flags,
 				   GthFileData           *file_data,
-				   const char            *attributes)
+				   const char            *attributes,
+				   GCancellable          *cancellable)
 {
 	void    *buffer = NULL;
 	gsize    size;
 	GError  *error = NULL;
 	GObject *metadata;
+	int      i;
 
 	if (((flags & GTH_METADATA_WRITE_FORCE_EMBEDDED) != GTH_METADATA_WRITE_FORCE_EMBEDDED) && ! eel_gconf_get_boolean (PREF_STORE_METADATA_IN_FILES, TRUE))
 		return;
@@ -129,7 +126,7 @@ gth_metadata_provider_exiv2_write (GthMetadataProvider   *self,
 	if (! exiv2_supports_writes (gth_file_data_get_mime_type (file_data)))
 		return;
 
-	if (! g_load_file_in_buffer (file_data->file, &buffer, &size, &error))
+	if (! g_load_file_in_buffer (file_data->file, &buffer, &size, cancellable, &error))
 		return;
 
 	metadata = g_file_info_get_attribute_object (file_data->info, "general::description");
@@ -146,7 +143,6 @@ gth_metadata_provider_exiv2_write (GthMetadataProvider   *self,
 			"Iptc::Application2::Caption",
 			NULL
 		};
-		int i;
 
 		for (i = 0; tags_to_remove[i] != NULL; i++)
 			g_file_info_remove_attribute (file_data->info, tags_to_remove[i]);
@@ -173,19 +169,8 @@ gth_metadata_provider_exiv2_write (GthMetadataProvider   *self,
 		}
 	}
 	else {
-		const char *tags_to_remove[] = {
-			"Exif::Image::ImageDescription",
-			"Xmp::tiff::ImageDescription",
-			"Iptc::Application2::Headline",
-			"Exif::Photo::UserComment",
-			"Xmp::dc::description",
-			"Iptc::Application2::Caption",
-			NULL
-		};
-		int i;
-
-		for (i = 0; tags_to_remove[i] != NULL; i++)
-			g_file_info_remove_attribute (file_data->info, tags_to_remove[i]);
+		for (i = 0; _DESCRIPTION_TAG_NAMES[i] != NULL; i++)
+			g_file_info_remove_attribute (file_data->info, _DESCRIPTION_TAG_NAMES[i]);
 	}
 
 	metadata = g_file_info_get_attribute_object (file_data->info, "general::title");
@@ -224,11 +209,38 @@ gth_metadata_provider_exiv2_write (GthMetadataProvider   *self,
 
 	metadata = g_file_info_get_attribute_object (file_data->info, "general::datetime");
 	if (metadata != NULL) {
-		g_object_set (metadata, "value-type", NULL, NULL);
-		g_file_info_set_attribute_object (file_data->info, "Exif::Image::DateTime", metadata);
+		GthMetadata *xmp_metadata = NULL;
+		GTimeVal     timeval;
+
+		if (_g_time_val_from_exif_date (gth_metadata_get_raw (GTH_METADATA (metadata)), &timeval)) {
+			char *xmp_format;
+
+			xmp_metadata = gth_metadata_new ();
+			xmp_format = _g_time_val_to_xmp_date (&timeval);
+			g_object_set (xmp_metadata,
+				      "raw", xmp_format,
+				      "formatted", gth_metadata_get_formatted (GTH_METADATA (metadata)),
+				      "value-type", NULL, /* use the default type as described in extensions/exiv2_tools/main.c */
+				      NULL);
+
+			g_free (xmp_format);
+		}
+
+		for (i = 0; _ORIGINAL_DATE_TAG_NAMES[i] != NULL; i++) {
+			if (g_str_has_prefix (_ORIGINAL_DATE_TAG_NAMES[i], "Xmp::")) {
+				if (xmp_metadata != NULL)
+					g_file_info_set_attribute_object (file_data->info, _ORIGINAL_DATE_TAG_NAMES[i], G_OBJECT (xmp_metadata));
+			}
+			else
+				g_file_info_set_attribute_object (file_data->info, _ORIGINAL_DATE_TAG_NAMES[i], metadata);
+		}
+
+		_g_object_unref (xmp_metadata);
+	}
+	else {
+		for (i = 0; _ORIGINAL_DATE_TAG_NAMES[i] != NULL; i++)
+			g_file_info_remove_attribute (file_data->info, _ORIGINAL_DATE_TAG_NAMES[i]);
 	}
-	else
-		g_file_info_remove_attribute (file_data->info, "Exif::Image::DateTime");
 
 	if (exiv2_write_metadata_to_buffer (&buffer,
 					    &size,
@@ -243,7 +255,7 @@ gth_metadata_provider_exiv2_write (GthMetadataProvider   *self,
 			      G_FILE_CREATE_NONE,
 			      buffer,
 			      size,
-			      NULL,
+			      cancellable,
 			      &error);
 
 		tmp_info = g_file_info_new ();
