@@ -34,10 +34,12 @@
 #include "gth-file-tool-negative.h"
 #include "gth-file-tool-redo.h"
 #include "gth-file-tool-resize.h"
+#include "gth-file-tool-rotate.h"
 #include "gth-file-tool-rotate-left.h"
 #include "gth-file-tool-rotate-right.h"
 #include "gth-file-tool-save.h"
 #include "gth-file-tool-save-as.h"
+#include "gth-file-tool-sharpen.h"
 #include "gth-file-tool-undo.h"
 
 
@@ -51,15 +53,17 @@ gthumb_extension_activate (void)
 
 	gth_main_register_type ("file-tools", GTH_TYPE_FILE_TOOL_ENHANCE);
 	gth_main_register_type ("file-tools", GTH_TYPE_FILE_TOOL_ADJUST_COLORS);
+	gth_main_register_type ("file-tools", GTH_TYPE_FILE_TOOL_SHARPEN);
 	gth_main_register_type ("file-tools", GTH_TYPE_FILE_TOOL_EQUALIZE);
 	gth_main_register_type ("file-tools", GTH_TYPE_FILE_TOOL_DESATURATE);
 	gth_main_register_type ("file-tools", GTH_TYPE_FILE_TOOL_NEGATIVE);
 
-	gth_main_register_type ("file-tools", GTH_TYPE_FILE_TOOL_MIRROR);
-	gth_main_register_type ("file-tools", GTH_TYPE_FILE_TOOL_FLIP);
 	gth_main_register_type ("file-tools", GTH_TYPE_FILE_TOOL_ROTATE_RIGHT);
 	gth_main_register_type ("file-tools", GTH_TYPE_FILE_TOOL_ROTATE_LEFT);
+	gth_main_register_type ("file-tools", GTH_TYPE_FILE_TOOL_MIRROR);
+	gth_main_register_type ("file-tools", GTH_TYPE_FILE_TOOL_FLIP);
 
+	gth_main_register_type ("file-tools", GTH_TYPE_FILE_TOOL_ROTATE);
 	gth_main_register_type ("file-tools", GTH_TYPE_FILE_TOOL_RESIZE);
 	gth_main_register_type ("file-tools", GTH_TYPE_FILE_TOOL_CROP);
 
