@@ -64,6 +64,7 @@ struct _GthMediaViewerPagePrivate {
 	GdkCursor      *cursor;
 	GdkCursor      *cursor_void;
 	gboolean        cursor_visible;
+	GthScreensaver *screensaver;
 };
 
 
@@ -621,12 +622,26 @@ update_progress_cb (gpointer user_data)
 }
 
 
+static void
+set_playing_state (GthMediaViewerPage *self,
+	           gboolean            playing)
+{
+	self->priv->playing = playing;
+	if (self->priv->playing)
+		gth_screensaver_inhibit (self->priv->screensaver,
+					 GTK_WIDGET (self->priv->browser),
+					 _("Playing video"));
+	else
+		gth_screensaver_uninhibit (self->priv->screensaver);
+}
+
+
 static void
 update_play_button (GthMediaViewerPage *self,
 		    GstState            new_state)
 {
 	if (! self->priv->playing && (new_state == GST_STATE_PLAYING)) {
-		self->priv->playing = TRUE;
+		set_playing_state (self, TRUE);
 		gtk_image_set_from_stock (GTK_IMAGE (GET_WIDGET ("button_play_image")), GTK_STOCK_MEDIA_PAUSE, GTK_ICON_SIZE_LARGE_TOOLBAR);
 		gtk_widget_set_tooltip_text (GET_WIDGET ("button_play_image"), _("Pause"));
 
@@ -636,7 +651,7 @@ update_play_button (GthMediaViewerPage *self,
 		update_playback_info (self);
 	}
 	else if (self->priv->playing && (new_state != GST_STATE_PLAYING)) {
-		self->priv->playing = FALSE;
+		set_playing_state (self, FALSE);
 		gtk_image_set_from_stock (GTK_IMAGE (GET_WIDGET ("button_play_image")), GTK_STOCK_MEDIA_PLAY, GTK_ICON_SIZE_LARGE_TOOLBAR);
 		gtk_widget_set_tooltip_text (GET_WIDGET ("button_play_image"), _("Play"));
 
@@ -751,7 +766,7 @@ remove_fullscreen_toolbar (GthMediaViewerPage *self)
 	if (self->priv->fullscreen_toolbar == NULL)
 		return;
 
-	if (self->priv->mediabar->parent == self->priv->fullscreen_toolbar) {
+	if (gtk_widget_get_parent (self->priv->mediabar) == self->priv->fullscreen_toolbar) {
 		g_object_ref (self->priv->mediabar);
 		gtk_container_remove (GTK_CONTAINER (self->priv->fullscreen_toolbar), self->priv->mediabar);
 		gtk_box_pack_start (GTK_BOX (self->priv->area_box), self->priv->mediabar, FALSE, FALSE, 0);
@@ -831,8 +846,8 @@ reset_player_state (GthMediaViewerPage *self)
         }
 
 	update_play_button (self, GST_STATE_NULL);
-	self->priv->playing = FALSE;
 	self->priv->rate = 1.0;
+	set_playing_state (self, FALSE);
 }
 
 
@@ -1029,9 +1044,6 @@ static gboolean
 gth_media_viewer_page_real_can_view (GthViewerPage *base,
 				     GthFileData   *file_data)
 {
-	GthMediaViewerPage *self;
-
-	self = (GthMediaViewerPage*) base;
 	g_return_val_if_fail (file_data != NULL, FALSE);
 
 	return _g_mime_type_is_video (gth_file_data_get_mime_type (file_data)) || _g_mime_type_is_audio (gth_file_data_get_mime_type (file_data));
@@ -1139,7 +1151,7 @@ gth_media_viewer_page_real_fullscreen (GthViewerPage *base,
 		gtk_container_set_border_width (GTK_CONTAINER (self->priv->fullscreen_toolbar), 0);
 	}
 
-	if (self->priv->mediabar->parent == self->priv->area_box) {
+	if (gtk_widget_get_parent (self->priv->mediabar) == self->priv->area_box) {
 		g_object_ref (self->priv->mediabar);
 		gtk_container_remove (GTK_CONTAINER (self->priv->area_box), self->priv->mediabar);
 		gtk_container_add (GTK_CONTAINER (self->priv->fullscreen_toolbar), self->priv->mediabar);
@@ -1268,6 +1280,10 @@ gth_media_viewer_page_finalize (GObject *obj)
 	}
 	_g_object_unref (self->priv->icon);
 	_g_object_unref (self->priv->file_data);
+	if (self->priv->screensaver != NULL) {
+		gth_screensaver_uninhibit (self->priv->screensaver);
+		g_object_unref (self->priv->screensaver);
+	}
 
 	G_OBJECT_CLASS (gth_media_viewer_page_parent_class)->finalize (obj);
 }
@@ -1316,6 +1332,7 @@ gth_media_viewer_page_instance_init (GthMediaViewerPage *self)
 	self->priv->video_fps_d = 0;
 	self->priv->icon = NULL;
 	self->priv->cursor_visible = TRUE;
+	self->priv->screensaver = gth_screensaver_new (NULL);
 }
 
 
