@@ -55,6 +55,7 @@
 #include <string.h>
 #include <glib.h>
 #include <stdio.h>
+#include <png.h>
 #define GDK_PIXBUF_ENABLE_BACKEND
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gconf/gconf.h>
@@ -72,9 +73,24 @@ struct _GnomeDesktopThumbnailFactoryPrivate {
 
   GMutex *lock;
 
+#ifdef HAVE_GNOME_3
+
+  GList *thumbnailers;
+  GHashTable *mime_types_map;
+  GList *monitors;
+
+  GSettings *settings;
+  gboolean loaded : 1;
+  gboolean disabled : 1;
+  gchar **disabled_types;
+
+#else /* ! HAVE_GNOME_3 */
+
   GHashTable *scripts_hash;
   guint thumbnailers_notify;
   guint reread_scheduled;
+
+#endif
 };
 
 static const char *appname = "gnome-thumbnail-factory";
@@ -146,11 +162,12 @@ size_prepared_cb (GdkPixbufLoader *loader,
 }
 
 static GdkPixbuf *
-_gdk_pixbuf_new_from_uri_at_scale (const char *uri,
-				   gint        width,
-				   gint        height,
-				   gboolean    preserve_aspect_ratio,
-				   gboolean    load_from_preview_icon)
+_gdk_pixbuf_new_from_uri_at_scale (const char   *uri,
+				   gint          width,
+				   gint          height,
+				   gboolean      preserve_aspect_ratio,
+				   gboolean      load_from_preview_icon,
+				   GCancellable *cancellable)
 {
 	gboolean                result;
 	char                    buffer[LOAD_BUFFER_SIZE];
@@ -176,7 +193,7 @@ _gdk_pixbuf_new_from_uri_at_scale (const char *uri,
 		file_info = g_file_query_info (file,
 					       G_FILE_ATTRIBUTE_PREVIEW_ICON,
 					       G_FILE_QUERY_INFO_NONE,
-					       NULL,  /* GCancellable */
+					       cancellable,  /* GCancellable */
 					       NULL); /* return location for GError */
 
 		if (file_info != NULL) {
@@ -187,7 +204,7 @@ _gdk_pixbuf_new_from_uri_at_scale (const char *uri,
 				input_stream = g_loadable_icon_load (G_LOADABLE_ICON (object),
 								     0,     /* size */
 								     NULL,  /* return location for type */
-								     NULL,  /* GCancellable */
+								     cancellable,  /* GCancellable */
 								     NULL); /* return location for GError */
 
 			g_object_unref (file_info);
@@ -217,7 +234,7 @@ _gdk_pixbuf_new_from_uri_at_scale (const char *uri,
 		bytes_read = g_input_stream_read (input_stream,
 						  buffer,
 						  sizeof (buffer),
-						  NULL,
+						  cancellable,
 						  NULL);
 		if (bytes_read == -1)
 			break;
@@ -272,45 +289,496 @@ _gdk_pixbuf_new_from_uri_at_scale (const char *uri,
 }
 
 
-static void
-gnome_desktop_thumbnail_factory_finalize (GObject *object)
+#ifdef HAVE_GNOME_3
+
+
+#define LOAD_BUFFER_SIZE 4096
+
+#define THUMBNAILER_ENTRY_GROUP "Thumbnailer Entry"
+#define THUMBNAILER_EXTENSION   ".thumbnailer"
+
+typedef struct {
+    volatile gint ref_count;
+
+    gchar *path;
+
+    gchar  *try_exec;
+    gchar  *command;
+    gchar **mime_types;
+} Thumbnailer;
+
+static Thumbnailer *
+thumbnailer_ref (Thumbnailer *thumb)
 {
-  GnomeDesktopThumbnailFactory *factory;
-  GnomeDesktopThumbnailFactoryPrivate *priv;
-  GConfClient *client;
+  g_return_val_if_fail (thumb != NULL, NULL);
+  g_return_val_if_fail (thumb->ref_count > 0, NULL);
 
-  factory = GNOME_DESKTOP_THUMBNAIL_FACTORY (object);
-
-  priv = factory->priv;
-
-  if (priv->reread_scheduled != 0) {
-    g_source_remove (priv->reread_scheduled);
-    priv->reread_scheduled = 0;
-  }
-
-  if (priv->thumbnailers_notify != 0) {
-    client = gconf_client_get_default ();
-    gconf_client_notify_remove (client, priv->thumbnailers_notify);
-    priv->thumbnailers_notify = 0;
-    g_object_unref (client);
-  }
-
-  if (priv->scripts_hash)
-    {
-      g_hash_table_destroy (priv->scripts_hash);
-      priv->scripts_hash = NULL;
-    }
-
-  if (priv->lock)
-    {
-      g_mutex_free (priv->lock);
-      priv->lock = NULL;
-    }
-
-  if (G_OBJECT_CLASS (parent_class)->finalize)
-    (* G_OBJECT_CLASS (parent_class)->finalize) (object);
+  g_atomic_int_inc (&thumb->ref_count);
+  return thumb;
 }
 
+static void
+thumbnailer_unref (Thumbnailer *thumb)
+{
+  g_return_if_fail (thumb != NULL);
+  g_return_if_fail (thumb->ref_count > 0);
+
+  if (g_atomic_int_dec_and_test (&thumb->ref_count))
+    {
+      g_free (thumb->path);
+      g_free (thumb->try_exec);
+      g_free (thumb->command);
+      g_strfreev (thumb->mime_types);
+
+      g_slice_free (Thumbnailer, thumb);
+    }
+}
+
+static Thumbnailer *
+thumbnailer_load (Thumbnailer *thumb)
+{
+  GKeyFile *key_file;
+  GError *error = NULL;
+
+  key_file = g_key_file_new ();
+  if (!g_key_file_load_from_file (key_file, thumb->path, 0, &error))
+    {
+      g_warning ("Failed to load thumbnailer from \"%s\": %s\n", thumb->path, error->message);
+      g_error_free (error);
+      thumbnailer_unref (thumb);
+      g_key_file_free (key_file);
+
+      return NULL;
+    }
+
+  if (!g_key_file_has_group (key_file, THUMBNAILER_ENTRY_GROUP))
+    {
+      g_warning ("Invalid thumbnailer: missing group \"%s\"\n", THUMBNAILER_ENTRY_GROUP);
+      thumbnailer_unref (thumb);
+      g_key_file_free (key_file);
+
+      return NULL;
+    }
+
+  thumb->command = g_key_file_get_string (key_file, THUMBNAILER_ENTRY_GROUP, "Exec", NULL);
+  if (!thumb->command)
+    {
+      g_warning ("Invalid thumbnailer: missing Exec key\n");
+      thumbnailer_unref (thumb);
+      g_key_file_free (key_file);
+
+      return NULL;
+    }
+
+  thumb->mime_types = g_key_file_get_string_list (key_file, THUMBNAILER_ENTRY_GROUP, "MimeType", NULL, NULL);
+  if (!thumb->mime_types)
+    {
+      g_warning ("Invalid thumbnailer: missing MimeType key\n");
+      thumbnailer_unref (thumb);
+      g_key_file_free (key_file);
+
+      return NULL;
+    }
+
+  thumb->try_exec = g_key_file_get_string (key_file, THUMBNAILER_ENTRY_GROUP, "TryExec", NULL);
+
+  g_key_file_free (key_file);
+
+  return thumb;
+}
+
+static Thumbnailer *
+thumbnailer_reload (Thumbnailer *thumb)
+{
+  g_return_val_if_fail (thumb != NULL, NULL);
+
+  g_free (thumb->command);
+  thumb->command = NULL;
+  g_strfreev (thumb->mime_types);
+  thumb->mime_types = NULL;
+  g_free (thumb->try_exec);
+  thumb->try_exec = NULL;
+
+  return thumbnailer_load (thumb);
+}
+
+static Thumbnailer *
+thumbnailer_new (const gchar *path)
+{
+  Thumbnailer *thumb;
+
+  thumb = g_slice_new0 (Thumbnailer);
+  thumb->ref_count = 1;
+  thumb->path = g_strdup (path);
+
+  return thumbnailer_load (thumb);
+}
+
+
+static gpointer
+init_thumbnailers_dirs (gpointer data)
+{
+  const gchar * const *data_dirs;
+  gchar **thumbs_dirs;
+  guint i, length;
+
+  data_dirs = g_get_system_data_dirs ();
+  length = g_strv_length ((char **) data_dirs);
+
+  thumbs_dirs = g_new (gchar *, length + 2);
+  thumbs_dirs[0] = g_build_filename (g_get_user_data_dir (), "thumbnailers", NULL);
+  for (i = 0; i < length; i++)
+    thumbs_dirs[i + 1] = g_build_filename (data_dirs[i], "thumbnailers", NULL);
+  thumbs_dirs[length + 1] = NULL;
+
+  return thumbs_dirs;
+}
+
+static const gchar * const *
+get_thumbnailers_dirs (void)
+{
+  static GOnce once_init = G_ONCE_INIT;
+  return g_once (&once_init, init_thumbnailers_dirs, NULL);
+}
+
+
+/* These should be called with the lock held */
+static void
+gnome_desktop_thumbnail_factory_register_mime_types (GnomeDesktopThumbnailFactory *factory,
+                                                     Thumbnailer                  *thumb)
+{
+  GnomeDesktopThumbnailFactoryPrivate *priv = factory->priv;
+  gint i;
+
+  for (i = 0; thumb->mime_types[i]; i++)
+    {
+      if (!g_hash_table_lookup (priv->mime_types_map, thumb->mime_types[i]))
+        g_hash_table_insert (priv->mime_types_map,
+                             g_strdup (thumb->mime_types[i]),
+                             thumbnailer_ref (thumb));
+    }
+}
+
+static void
+gnome_desktop_thumbnail_factory_add_thumbnailer (GnomeDesktopThumbnailFactory *factory,
+                                                 Thumbnailer                  *thumb)
+{
+  GnomeDesktopThumbnailFactoryPrivate *priv = factory->priv;
+
+  gnome_desktop_thumbnail_factory_register_mime_types (factory, thumb);
+  priv->thumbnailers = g_list_prepend (priv->thumbnailers, thumb);
+}
+
+
+static gboolean
+gnome_desktop_thumbnail_factory_is_disabled (GnomeDesktopThumbnailFactory *factory,
+                                             const gchar                  *mime_type)
+{
+  GnomeDesktopThumbnailFactoryPrivate *priv = factory->priv;
+  guint i;
+
+  if (priv->disabled)
+    return TRUE;
+
+  if (!priv->disabled_types)
+    return FALSE;
+
+  for (i = 0; priv->disabled_types[i]; i++)
+    {
+      if (g_strcmp0 (priv->disabled_types[i], mime_type) == 0)
+        return TRUE;
+    }
+
+  return FALSE;
+}
+
+
+static gboolean
+remove_thumbnailer_from_mime_type_map (gchar       *key,
+                                       Thumbnailer *value,
+                                       gchar       *path)
+{
+  return (strcmp (value->path, path) == 0);
+}
+
+
+static void
+update_or_create_thumbnailer (GnomeDesktopThumbnailFactory *factory,
+                              const gchar                  *path)
+{
+  GnomeDesktopThumbnailFactoryPrivate *priv = factory->priv;
+  GList *l;
+  Thumbnailer *thumb;
+  gboolean found = FALSE;
+
+  g_mutex_lock (priv->lock);
+
+  for (l = priv->thumbnailers; l && !found; l = g_list_next (l))
+    {
+      thumb = (Thumbnailer *)l->data;
+
+      if (strcmp (thumb->path, path) == 0)
+        {
+          found = TRUE;
+
+          /* First remove the mime_types associated to this thumbnailer */
+          g_hash_table_foreach_remove (priv->mime_types_map,
+                                       (GHRFunc)remove_thumbnailer_from_mime_type_map,
+                                       (gpointer)path);
+          if (!thumbnailer_reload (thumb))
+              priv->thumbnailers = g_list_delete_link (priv->thumbnailers, l);
+          else
+              gnome_desktop_thumbnail_factory_register_mime_types (factory, thumb);
+        }
+    }
+
+  if (!found)
+    {
+      thumb = thumbnailer_new (path);
+      if (thumb)
+        gnome_desktop_thumbnail_factory_add_thumbnailer (factory, thumb);
+    }
+
+  g_mutex_unlock (priv->lock);
+}
+
+
+static void
+remove_thumbnailer (GnomeDesktopThumbnailFactory *factory,
+                    const gchar                  *path)
+{
+  GnomeDesktopThumbnailFactoryPrivate *priv = factory->priv;
+  GList *l;
+  Thumbnailer *thumb;
+
+  g_mutex_lock (priv->lock);
+
+  for (l = priv->thumbnailers; l; l = g_list_next (l))
+    {
+      thumb = (Thumbnailer *)l->data;
+
+      if (strcmp (thumb->path, path) == 0)
+        {
+          priv->thumbnailers = g_list_delete_link (priv->thumbnailers, l);
+          g_hash_table_foreach_remove (priv->mime_types_map,
+                                       (GHRFunc)remove_thumbnailer_from_mime_type_map,
+                                       (gpointer)path);
+          thumbnailer_unref (thumb);
+
+          break;
+        }
+    }
+
+  g_mutex_unlock (priv->lock);
+}
+
+
+static void
+thumbnailers_directory_changed (GFileMonitor                 *monitor,
+                                GFile                        *file,
+                                GFile                        *other_file,
+                                GFileMonitorEvent             event_type,
+                                GnomeDesktopThumbnailFactory *factory)
+{
+  gchar *path;
+
+  switch (event_type)
+    {
+    case G_FILE_MONITOR_EVENT_CREATED:
+    case G_FILE_MONITOR_EVENT_CHANGED:
+    case G_FILE_MONITOR_EVENT_DELETED:
+      path = g_file_get_path (file);
+      if (!g_str_has_suffix (path, THUMBNAILER_EXTENSION))
+        {
+          g_free (path);
+          return;
+        }
+
+      if (event_type == G_FILE_MONITOR_EVENT_DELETED)
+        remove_thumbnailer (factory, path);
+      else
+        update_or_create_thumbnailer (factory, path);
+
+      g_free (path);
+      break;
+    default:
+      break;
+    }
+}
+
+
+static void
+gnome_desktop_thumbnail_factory_load_thumbnailers (GnomeDesktopThumbnailFactory *factory)
+{
+  GnomeDesktopThumbnailFactoryPrivate *priv = factory->priv;
+  const gchar * const *dirs;
+  guint i;
+
+  if (priv->loaded)
+    return;
+
+  dirs = get_thumbnailers_dirs ();
+  for (i = 0; dirs[i]; i++)
+    {
+      const gchar *path = dirs[i];
+      GDir *dir;
+      GFile *dir_file;
+      GFileMonitor *monitor;
+      const gchar *dirent;
+
+      dir = g_dir_open (path, 0, NULL);
+      if (!dir)
+        continue;
+
+      /* Monitor dir */
+      dir_file = g_file_new_for_path (path);
+      monitor = g_file_monitor_directory (dir_file,
+                                          G_FILE_MONITOR_NONE,
+                                          NULL, NULL);
+      if (monitor)
+        {
+          g_signal_connect (monitor, "changed",
+                            G_CALLBACK (thumbnailers_directory_changed),
+                            factory);
+          priv->monitors = g_list_prepend (priv->monitors, monitor);
+        }
+      g_object_unref (dir_file);
+
+      while ((dirent = g_dir_read_name (dir)))
+        {
+          Thumbnailer *thumb;
+          gchar       *filename;
+
+          if (!g_str_has_suffix (dirent, THUMBNAILER_EXTENSION))
+            continue;
+
+          filename = g_build_filename (path, dirent, NULL);
+          thumb = thumbnailer_new (filename);
+          g_free (filename);
+
+          if (thumb)
+            gnome_desktop_thumbnail_factory_add_thumbnailer (factory, thumb);
+        }
+
+      g_dir_close (dir);
+    }
+
+  priv->loaded = TRUE;
+}
+
+
+static void
+external_thumbnailers_disabled_all_changed_cb (GSettings                    *settings,
+                                               const gchar                  *key,
+                                               GnomeDesktopThumbnailFactory *factory)
+{
+	g_mutex_lock (factory->priv->lock);
+
+	factory->priv->disabled = g_settings_get_boolean (factory->priv->settings, "disable-all");
+	if (factory->priv->disabled) {
+		g_strfreev (factory->priv->disabled_types);
+		factory->priv->disabled_types = NULL;
+	}
+	else {
+		factory->priv->disabled_types = g_settings_get_strv (factory->priv->settings, "disable");
+		gnome_desktop_thumbnail_factory_load_thumbnailers (factory);
+	}
+
+	g_mutex_unlock (factory->priv->lock);
+}
+
+static void
+external_thumbnailers_disabled_changed_cb (GSettings                    *settings,
+                                           const gchar                  *key,
+                                           GnomeDesktopThumbnailFactory *factory)
+{
+	g_mutex_lock (factory->priv->lock);
+
+	if (factory->priv->disabled)
+		return;
+	g_strfreev (factory->priv->disabled_types);
+	factory->priv->disabled_types = g_settings_get_strv (factory->priv->settings, "disable");
+
+	g_mutex_unlock (factory->priv->lock);
+}
+
+
+static void
+gnome_desktop_thumbnail_factory_init_scripts (GnomeDesktopThumbnailFactory *factory)
+{
+	factory->priv->mime_types_map = g_hash_table_new_full (g_str_hash,
+							       g_str_equal,
+							       (GDestroyNotify)g_free,
+							       (GDestroyNotify)thumbnailer_unref);
+	factory->priv->settings = g_settings_new ("org.gnome.desktop.thumbnailers");
+	factory->priv->disabled = g_settings_get_boolean (factory->priv->settings, "disable-all");
+	if (! factory->priv->disabled)
+		factory->priv->disabled_types = g_settings_get_strv (factory->priv->settings, "disable");
+	g_signal_connect (factory->priv->settings,
+			  "changed::disable-all",
+			  G_CALLBACK (external_thumbnailers_disabled_all_changed_cb),
+			  factory);
+	g_signal_connect (factory->priv->settings,
+			  "changed::disable",
+			  G_CALLBACK (external_thumbnailers_disabled_changed_cb),
+			  factory);
+
+	if (! factory->priv->disabled)
+		gnome_desktop_thumbnail_factory_load_thumbnailers (factory);
+}
+
+
+static void
+gnome_desktop_thumbnail_factory_finalize_scripts (GnomeDesktopThumbnailFactory *factory)
+{
+	if (factory->priv->thumbnailers) {
+		g_list_free_full (factory->priv->thumbnailers, (GDestroyNotify)thumbnailer_unref);
+		factory->priv->thumbnailers = NULL;
+	}
+
+	if (factory->priv->mime_types_map) {
+		g_hash_table_destroy (factory->priv->mime_types_map);
+		factory->priv->mime_types_map = NULL;
+	}
+
+	if (factory->priv->monitors) {
+		g_list_free_full (factory->priv->monitors, (GDestroyNotify)g_object_unref);
+		factory->priv->monitors = NULL;
+	}
+
+	if (factory->priv->disabled_types) {
+		g_strfreev (factory->priv->disabled_types);
+		factory->priv->disabled_types = NULL;
+	}
+
+	if (factory->priv->settings) {
+		g_object_unref (factory->priv->settings);
+		factory->priv->settings = NULL;
+	}
+}
+
+
+static char *
+gnome_desktop_thumbnail_factory_get_script (GnomeDesktopThumbnailFactory *factory,
+					    const char                   *mime_type)
+{
+	char *script = NULL;
+
+	if (!gnome_desktop_thumbnail_factory_is_disabled (factory, mime_type)) {
+		Thumbnailer *thumb;
+
+		thumb = g_hash_table_lookup (factory->priv->mime_types_map, mime_type);
+		if (thumb)
+			script = g_strdup (thumb->command);
+	}
+
+	return script;
+}
+
+
+#else /* ! HAVE_GNOME_3 */
+
+
 /* Must be called on main thread */
 static GHashTable *
 read_scripts (void)
@@ -406,6 +874,7 @@ gnome_desktop_thumbnail_factory_reread_scripts (GnomeDesktopThumbnailFactory *fa
   g_mutex_unlock (priv->lock);
 }
 
+
 static gboolean
 reread_idle_callback (gpointer user_data)
 {
@@ -442,34 +911,98 @@ schedule_reread (GConfClient* client,
 }
 
 
+static void
+gnome_desktop_thumbnail_factory_init_scripts (GnomeDesktopThumbnailFactory *factory)
+{
+	GConfClient *client;
+
+	factory->priv->scripts_hash = NULL;
+
+	client = gconf_client_get_default ();
+	gconf_client_add_dir (client,
+			      "/desktop/gnome/thumbnailers",
+			      GCONF_CLIENT_PRELOAD_RECURSIVE, NULL);
+
+	gnome_desktop_thumbnail_factory_reread_scripts (factory);
+
+	factory->priv->thumbnailers_notify = gconf_client_notify_add (client, "/desktop/gnome/thumbnailers",
+								      schedule_reread, factory, NULL,
+								      NULL);
+
+	g_object_unref (G_OBJECT (client));
+}
+
+
+static void
+gnome_desktop_thumbnail_factory_finalize_scripts (GnomeDesktopThumbnailFactory *factory)
+{
+	if (factory->priv->scripts_hash) {
+		g_hash_table_destroy (factory->priv->scripts_hash);
+		factory->priv->scripts_hash = NULL;
+	}
+
+	if (factory->priv->reread_scheduled != 0) {
+		g_source_remove (factory->priv->reread_scheduled);
+		factory->priv->reread_scheduled = 0;
+	}
+
+	if (factory->priv->thumbnailers_notify != 0) {
+		GConfClient *client;
+
+		client = gconf_client_get_default ();
+		gconf_client_notify_remove (client, factory->priv->thumbnailers_notify);
+		factory->priv->thumbnailers_notify = 0;
+		g_object_unref (client);
+	}
+}
+
+
+static char *
+gnome_desktop_thumbnail_factory_get_script (GnomeDesktopThumbnailFactory *factory,
+					    const char                   *mime_type)
+{
+	char *script = NULL;
+
+	if (factory->priv->scripts_hash != NULL) {
+		script = g_hash_table_lookup (factory->priv->scripts_hash, mime_type);
+		if (script)
+			script = g_strdup (script);
+	}
+
+	return script;
+}
+
+#endif
+
+
+static void
+gnome_desktop_thumbnail_factory_finalize (GObject *object)
+{
+	GnomeDesktopThumbnailFactory *factory;
+
+	factory = GNOME_DESKTOP_THUMBNAIL_FACTORY (object);
+
+	gnome_desktop_thumbnail_factory_finalize_scripts (factory);
+
+	if (factory->priv->lock) {
+		g_mutex_free (factory->priv->lock);
+		factory->priv->lock = NULL;
+	}
+
+	if (G_OBJECT_CLASS (parent_class)->finalize)
+		(* G_OBJECT_CLASS (parent_class)->finalize) (object);
+}
+
+
 static void
 gnome_desktop_thumbnail_factory_init (GnomeDesktopThumbnailFactory *factory)
 {
-  GConfClient *client;
-  GnomeDesktopThumbnailFactoryPrivate *priv;
+	factory->priv = GNOME_DESKTOP_THUMBNAIL_FACTORY_GET_PRIVATE (factory);
+	factory->priv->size = GNOME_DESKTOP_THUMBNAIL_SIZE_NORMAL;
+	factory->priv->lock = g_mutex_new ();
 
-  factory->priv = GNOME_DESKTOP_THUMBNAIL_FACTORY_GET_PRIVATE (factory);
+	gnome_desktop_thumbnail_factory_init_scripts (factory);
 
-  priv = factory->priv;
-
-  priv->size = GNOME_DESKTOP_THUMBNAIL_SIZE_NORMAL;
-
-  priv->scripts_hash = NULL;
-
-  priv->lock = g_mutex_new ();
-
-  client = gconf_client_get_default ();
-  gconf_client_add_dir (client,
-			"/desktop/gnome/thumbnailers",
-			GCONF_CLIENT_PRELOAD_RECURSIVE, NULL);
-
-  gnome_desktop_thumbnail_factory_reread_scripts (factory);
-
-  priv->thumbnailers_notify = gconf_client_notify_add (client, "/desktop/gnome/thumbnailers",
-						       schedule_reread, factory, NULL,
-						       NULL);
-
-  g_object_unref (G_OBJECT (client));
 }
 
 static void
@@ -524,50 +1057,38 @@ gnome_desktop_thumbnail_factory_new (GnomeDesktopThumbnailSize size)
  **/
 char *
 gnome_desktop_thumbnail_factory_lookup (GnomeDesktopThumbnailFactory *factory,
-					const char            *uri,
-					time_t                 mtime)
+					const char                   *uri,
+					time_t                        mtime)
 {
-  GnomeDesktopThumbnailFactoryPrivate *priv = factory->priv;
-  char *path, *file;
-  GChecksum *checksum;
-  guint8 digest[16];
-  gsize digest_len = sizeof (digest);
-  GdkPixbuf *pixbuf;
-  gboolean res;
+	GChecksum *checksum;
+	guint8     digest[16];
+	gsize      digest_len = sizeof (digest);
+	char      *file;
+	char      *path;
 
-  g_return_val_if_fail (uri != NULL, NULL);
+	g_return_val_if_fail (uri != NULL, NULL);
 
-  res = FALSE;
+	checksum = g_checksum_new (G_CHECKSUM_MD5);
+	g_checksum_update (checksum, (const guchar *) uri, strlen (uri));
+	g_checksum_get_digest (checksum, digest, &digest_len);
+	g_assert (digest_len == 16);
 
-  checksum = g_checksum_new (G_CHECKSUM_MD5);
-  g_checksum_update (checksum, (const guchar *) uri, strlen (uri));
+	file = g_strconcat (g_checksum_get_string (checksum), ".png", NULL);
+	path = g_build_filename (g_get_home_dir (),
+				 ".thumbnails",
+				 (factory->priv->size == GNOME_DESKTOP_THUMBNAIL_SIZE_NORMAL)?"normal":"large",
+				 file,
+				 NULL);
 
-  g_checksum_get_digest (checksum, digest, &digest_len);
-  g_assert (digest_len == 16);
+	if (! gnome_desktop_thumbnail_is_valid (path, uri, mtime)) {
+		g_free (path);
+		path = NULL;
+	}
 
-  file = g_strconcat (g_checksum_get_string (checksum), ".png", NULL);
+	g_free (file);
+	g_checksum_free (checksum);
 
-  path = g_build_filename (g_get_home_dir (),
-			   ".thumbnails",
-			   (priv->size == GNOME_DESKTOP_THUMBNAIL_SIZE_NORMAL)?"normal":"large",
-			   file,
-			   NULL);
-  g_free (file);
-
-  pixbuf = gdk_pixbuf_new_from_file (path, NULL);
-  if (pixbuf != NULL)
-    {
-      res = gnome_desktop_thumbnail_is_valid (pixbuf, uri, mtime);
-      g_object_unref (pixbuf);
-    }
-
-  g_checksum_free (checksum);
-
-  if (res)
-    return path;
-
-  g_free (path);
-  return FALSE;
+	return path;
 }
 
 /**
@@ -588,45 +1109,34 @@ gnome_desktop_thumbnail_factory_lookup (GnomeDesktopThumbnailFactory *factory,
  **/
 gboolean
 gnome_desktop_thumbnail_factory_has_valid_failed_thumbnail (GnomeDesktopThumbnailFactory *factory,
-							    const char            *uri,
-							    time_t                 mtime)
+							    const char                   *uri,
+							    time_t                        mtime)
 {
-  char *path, *file;
-  GdkPixbuf *pixbuf;
-  gboolean res;
-  GChecksum *checksum;
-  guint8 digest[16];
-  gsize digest_len = sizeof (digest);
+	GChecksum *checksum;
+	guint8     digest[16];
+	gsize      digest_len = sizeof (digest);
+	char      *path;
+	char      *file;
+	gboolean   res;
 
-  checksum = g_checksum_new (G_CHECKSUM_MD5);
-  g_checksum_update (checksum, (const guchar *) uri, strlen (uri));
+	checksum = g_checksum_new (G_CHECKSUM_MD5);
+	g_checksum_update (checksum, (const guchar *) uri, strlen (uri));
+	g_checksum_get_digest (checksum, digest, &digest_len);
+	g_assert (digest_len == 16);
 
-  g_checksum_get_digest (checksum, digest, &digest_len);
-  g_assert (digest_len == 16);
+	file = g_strconcat (g_checksum_get_string (checksum), ".png", NULL);
+	path = g_build_filename (g_get_home_dir (),
+				 ".thumbnails/fail",
+				 appname,
+				 file,
+				 NULL);
+	res = gnome_desktop_thumbnail_is_valid (path, uri, mtime);
 
-  res = FALSE;
+	g_free (path);
+	g_free (file);
+	g_checksum_free (checksum);
 
-  file = g_strconcat (g_checksum_get_string (checksum), ".png", NULL);
-
-  path = g_build_filename (g_get_home_dir (),
-			   ".thumbnails/fail",
-			   appname,
-			   file,
-			   NULL);
-  g_free (file);
-
-  pixbuf = gdk_pixbuf_new_from_file (path, NULL);
-  g_free (path);
-
-  if (pixbuf)
-    {
-      res = gnome_desktop_thumbnail_is_valid (pixbuf, uri, mtime);
-      g_object_unref (pixbuf);
-    }
-
-  g_checksum_free (checksum);
-
-  return res;
+	return res;
 }
 
 static char *
@@ -701,8 +1211,9 @@ expand_thumbnailing_script (const char *script,
 
 GdkPixbuf *
 gnome_desktop_thumbnail_factory_generate_no_script (GnomeDesktopThumbnailFactory *factory,
-						    const char            *uri,
-						    const char            *mime_type)
+						    const char                   *uri,
+						    const char                   *mime_type,
+						    GCancellable                 *cancellable)
 {
   GdkPixbuf *pixbuf, *scaled, *tmp_pixbuf;
   int width, height, size;
@@ -721,7 +1232,7 @@ gnome_desktop_thumbnail_factory_generate_no_script (GnomeDesktopThumbnailFactory
     size = 256;
 
   /* Check for preview::icon first */
-  pixbuf = _gdk_pixbuf_new_from_uri_at_scale (uri, size, size, TRUE, TRUE);
+  pixbuf = _gdk_pixbuf_new_from_uri_at_scale (uri, size, size, TRUE, TRUE, cancellable);
 
   /* ...then use a registered thumbnail generator (the exiv2 extension tries
    * to read the embedded thumbnail) */
@@ -730,7 +1241,7 @@ gnome_desktop_thumbnail_factory_generate_no_script (GnomeDesktopThumbnailFactory
 
   /* ...lastly try the whole file */
   if (pixbuf == NULL)
-    pixbuf = _gdk_pixbuf_new_from_uri_at_scale (uri, size, size, TRUE, FALSE);
+    pixbuf = _gdk_pixbuf_new_from_uri_at_scale (uri, size, size, TRUE, FALSE, cancellable);
 
   if (pixbuf == NULL)
     return NULL;
@@ -787,11 +1298,13 @@ gnome_desktop_thumbnail_factory_generate_no_script (GnomeDesktopThumbnailFactory
     {
       g_snprintf (dimension, sizeof (dimension), "%i", original_width);
       gdk_pixbuf_set_option (pixbuf, "tEXt::Thumb::Image::Width", dimension);
+      g_object_set_data (G_OBJECT (pixbuf), "gnome-original-width", GINT_TO_POINTER (original_width));
     }
   if (original_height > 0)
     {
       g_snprintf (dimension, sizeof (dimension), "%i", original_height);
       gdk_pixbuf_set_option (pixbuf, "tEXt::Thumb::Image::Height", dimension);
+      g_object_set_data (G_OBJECT (pixbuf), "gnome-original-height", GINT_TO_POINTER (original_height));
     }
 
   return pixbuf;
@@ -823,11 +1336,7 @@ gnome_desktop_thumbnail_factory_generate_from_script (GnomeDesktopThumbnailFacto
 
 	script = NULL;
 	g_mutex_lock (factory->priv->lock);
-	if (factory->priv->scripts_hash != NULL) {
-		script = g_hash_table_lookup (factory->priv->scripts_hash, mime_type);
-		if (script)
-			script = g_strdup (script);
-	}
+	script = gnome_desktop_thumbnail_factory_get_script (factory, mime_type);
 	g_mutex_unlock (factory->priv->lock);
 
 	if (script == NULL) {
@@ -842,8 +1351,17 @@ gnome_desktop_thumbnail_factory_generate_from_script (GnomeDesktopThumbnailFacto
 
 	expanded_script = expand_thumbnailing_script (script, size, uri, *tmpname);
 	if (g_shell_parse_argv (expanded_script, &argc, &argv, error))
-		if (g_spawn_async (NULL, argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL, pid, error))
+		if (g_spawn_async (g_get_tmp_dir (),
+				   argv,
+				   NULL,
+				   G_SPAWN_DO_NOT_REAP_CHILD | G_SPAWN_SEARCH_PATH,
+				   NULL,
+				   NULL,
+				   pid,
+				   error))
+		{
 			retval = TRUE;
+		}
 
 	g_free (expanded_script);
 	g_free (script);
@@ -1200,9 +1718,104 @@ gnome_desktop_thumbnail_path_for_uri (const char         *uri,
 }
 
 
+/* This function taken from gdk-pixbuf:
+ *
+ * Copyright (C) 1999 Mark Crichton
+ * Copyright (C) 1999 The Free Software Foundation
+ *
+ * Authors: Mark Crichton <crichton@gimp.org>
+ *          Federico Mena-Quintero <federico@gimp.org>
+ *
+ * Released under the LGPL version 2 of the License,
+ * or (at your option) any later version.
+ *
+ **/
+static gboolean
+png_text_to_pixbuf_option (png_text   text_ptr,
+                           gchar    **key,
+                           gchar    **value)
+{
+        gboolean is_ascii = TRUE;
+        int i;
+
+        /* Avoid loading iconv if the text is plain ASCII */
+        for (i = 0; i < text_ptr.text_length; i++)
+                if (text_ptr.text[i] & 0x80) {
+                        is_ascii = FALSE;
+                        break;
+                }
+
+        if (is_ascii) {
+                *value = g_strdup (text_ptr.text);
+        } else {
+                *value = g_convert (text_ptr.text, -1,
+                                     "UTF-8", "ISO-8859-1",
+                                     NULL, NULL, NULL);
+        }
+
+        if (*value) {
+                *key = g_strconcat ("tEXt::", text_ptr.key, NULL);
+                return TRUE;
+        } else {
+                g_warning ("Couldn't convert text chunk value to UTF-8.");
+                *key = NULL;
+                return FALSE;
+        }
+}
+
+
+static GHashTable *
+read_png_options (const char *thumbnail_filename)
+{
+	GHashTable  *options;
+	png_structp  png_ptr;
+        png_infop    info_ptr;
+        FILE        *f;
+        png_textp    text_ptr;
+	int          num_texts;
+
+	options = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
+
+	png_ptr = png_create_read_struct (PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
+	if (png_ptr == NULL)
+		return options;
+
+	info_ptr = png_create_info_struct (png_ptr);
+	if (info_ptr == NULL) {
+		png_destroy_read_struct (&png_ptr, NULL, NULL);
+		return options;
+	}
+
+	f = fopen (thumbnail_filename, "r");
+	if (f == NULL) {
+		png_destroy_read_struct (&png_ptr, NULL, NULL);
+		return options;
+	}
+
+	png_init_io (png_ptr, f);
+	png_read_info (png_ptr, info_ptr);
+
+	if (png_get_text (png_ptr, info_ptr, &text_ptr, &num_texts)) {
+		int i;
+		for (i = 0; i < num_texts; i++) {
+			char *key;
+			char *value;
+
+			if (png_text_to_pixbuf_option (text_ptr[i], &key, &value))
+				g_hash_table_insert (options, key, value);
+		}
+	}
+
+	png_destroy_read_struct (&png_ptr, &info_ptr, NULL);
+	fclose (f);
+
+	return options;
+}
+
+
 /**
  * gnome_desktop_thumbnail_is_valid:
- * @pixbuf: an loaded thumbnail #GdkPixbuf
+ * @thumbnail_filename: the png file that contains the thumbnail
  * @uri: a uri
  * @mtime: the mtime
  *
@@ -1214,25 +1827,25 @@ gnome_desktop_thumbnail_path_for_uri (const char         *uri,
  * Since: 2.2
  **/
 gboolean
-gnome_desktop_thumbnail_is_valid (GdkPixbuf          *pixbuf,
-				  const char         *uri,
-				  time_t              mtime)
+gnome_desktop_thumbnail_is_valid (const char *thumbnail_filename,
+				  const char *uri,
+				  time_t      mtime)
 {
-  const char *thumb_uri, *thumb_mtime_str;
-  time_t thumb_mtime;
+	gboolean    is_valid = FALSE;
+	GHashTable *png_options;
+	const char *thumb_uri;
 
-  thumb_uri = gdk_pixbuf_get_option (pixbuf, "tEXt::Thumb::URI");
-  if (!thumb_uri)
-    return FALSE;
-  if (strcmp (uri, thumb_uri) != 0)
-    return FALSE;
+	png_options = read_png_options (thumbnail_filename);
+	thumb_uri = g_hash_table_lookup (png_options, "tEXt::Thumb::URI");
+	if (g_strcmp0 (uri, thumb_uri) == 0) {
+		const char *thumb_mtime_str;
 
-  thumb_mtime_str = gdk_pixbuf_get_option (pixbuf, "tEXt::Thumb::MTime");
-  if (!thumb_mtime_str)
-    return FALSE;
-  thumb_mtime = atol (thumb_mtime_str);
-  if (mtime != thumb_mtime)
-    return FALSE;
+		thumb_mtime_str = g_hash_table_lookup (png_options, "tEXt::Thumb::MTime");
+		if ((thumb_mtime_str != NULL) && (mtime == atol (thumb_mtime_str)))
+			is_valid = TRUE;
+	}
 
-  return TRUE;
+	g_hash_table_unref (png_options);
+
+	return is_valid;
 }
