@@ -68,7 +68,6 @@ struct _GthFileStorePrivate
 	gboolean             load_thumbs;
 	GthTest             *filter;
 	GList               *queue;
-	int                  queue_size;
 	GthFileDataCompFunc  cmp_func;
 	gboolean             inverse_sort;
 	gboolean             update_filter;
@@ -164,7 +163,6 @@ _gth_file_store_clear_queue (GthFileStore *file_store)
 {
 	g_list_free (file_store->priv->queue);
 	file_store->priv->queue = NULL;
-	file_store->priv->queue_size = 0;
 }
 
 
@@ -255,6 +253,7 @@ gth_file_store_init (GthFileStore *file_store)
 		column_type[GTH_FILE_STORE_IS_ICON_COLUMN] = G_TYPE_BOOLEAN;
 		column_type[GTH_FILE_STORE_FILENAME_COLUMN] = G_TYPE_STRING;
 		column_type[GTH_FILE_STORE_METADATA_COLUMN] = G_TYPE_STRING;
+		column_type[GTH_FILE_STORE_METADATA_VISIBLE_COLUMN] = G_TYPE_BOOLEAN;
 		column_type[GTH_FILE_STORE_CHECKED_COLUMN] = G_TYPE_BOOLEAN;
 	}
 }
@@ -375,6 +374,10 @@ gth_file_store_get_value (GtkTreeModel *tree_model,
 		g_value_init (value, G_TYPE_STRING);
 		g_value_set_string (value, row->metadata);
 		break;
+	case GTH_FILE_STORE_METADATA_VISIBLE_COLUMN:
+		g_value_init (value, G_TYPE_BOOLEAN);
+		g_value_set_boolean (value, (row->metadata != NULL) && ! g_str_equal (row->metadata, ""));
+		break;
 	case GTH_FILE_STORE_CHECKED_COLUMN:
 		g_value_init (value, G_TYPE_BOOLEAN);
 		g_value_set_boolean (value, row->checked);
@@ -797,7 +800,8 @@ _gth_file_store_hide_row (GthFileStore *file_store,
 
 static void
 _gth_file_store_update_visibility (GthFileStore *file_store,
-				   GList        *add_queue)
+				   GList        *add_queue,
+				   int           position)
 {
 	GthFileRow **all_rows = NULL;
 	guint        all_rows_n = 0;
@@ -811,7 +815,6 @@ _gth_file_store_update_visibility (GthFileStore *file_store,
 	GthFileData *file;
 	int          j, k;
 	gboolean     row_deleted;
-	gboolean     row_inserted;
 
 
 #ifdef DEBUG_FILE_STORE
@@ -822,11 +825,21 @@ g_print ("UPDATE VISIBILITY\n");
 
 	all_rows_n = file_store->priv->tot_rows + g_list_length (add_queue);
 	all_rows = g_new (GthFileRow *, all_rows_n);
-	for (i = 0; i < file_store->priv->tot_rows; i++)
-		all_rows[i] = _gth_file_row_copy (file_store->priv->all_rows[i]);
 
+	/* append to the end if position is -1 */
+
+	if (position == -1)
+		position = file_store->priv->tot_rows;
+
+	/* insert the new rows at position */
+
+	j = 0;
+	for (i = 0; i < position; i++)
+		all_rows[j++] = _gth_file_row_copy (file_store->priv->all_rows[i]);
 	for (scan = add_queue; scan; scan = scan->next)
-		all_rows[i++] = (GthFileRow *) scan->data;
+		all_rows[j++] = (GthFileRow *) scan->data;
+	for (i = position; i < file_store->priv->tot_rows; i++)
+		all_rows[j++] = _gth_file_row_copy (file_store->priv->all_rows[i]);
 
 	old_rows_n = file_store->priv->num_rows;
 	old_rows = g_new (GthFileRow *, old_rows_n);
@@ -1014,7 +1027,6 @@ g_print ("\n");
 
 	/* add the new files */
 
-	row_inserted = FALSE;
 	for (i = 0; i < new_rows_n; i++) {
 		GtkTreePath *path;
 		GtkTreeIter  iter;
@@ -1041,8 +1053,6 @@ g_print ("  INSERT: %d\n", i);
 		gth_file_store_get_iter (GTK_TREE_MODEL (file_store), &iter, path);
 		gtk_tree_model_row_inserted (GTK_TREE_MODEL (file_store), path, &iter);
 		gtk_tree_path_free (path);
-
-		row_inserted = TRUE;
 	}
 
 	g_signal_emit (file_store, gth_file_store_signals[VISIBILITY_CHANGED], 0);
@@ -1067,7 +1077,7 @@ gth_file_store_set_filter (GthFileStore *file_store,
 	else
 		file_store->priv->filter = gth_test_new ();
 
-	_gth_file_store_update_visibility (file_store, NULL);
+	_gth_file_store_update_visibility (file_store, NULL, -1);
 }
 
 
@@ -1394,10 +1404,11 @@ gth_file_store_add (GthFileStore *file_store,
 		    GdkPixbuf    *thumbnail,
 		    gboolean      is_icon,
 		    const char   *metadata,
-		    gboolean      checked)
+		    gboolean      checked,
+		    int           position)
 {
 	gth_file_store_queue_add (file_store, file, thumbnail, is_icon, metadata, checked);
-	gth_file_store_exec_add (file_store);
+	gth_file_store_exec_add (file_store, position);
 }
 
 
@@ -1419,14 +1430,16 @@ gth_file_store_queue_add (GthFileStore *file_store,
 	_gth_file_row_set_metadata (row, metadata);
 	row->is_icon = is_icon;
 	row->checked = checked;
+
 	file_store->priv->queue = g_list_prepend (file_store->priv->queue, row);
 }
 
 
 void
-gth_file_store_exec_add (GthFileStore *file_store)
+gth_file_store_exec_add (GthFileStore *file_store,
+			 int           position)
 {
-	_gth_file_store_update_visibility (file_store, file_store->priv->queue);
+	_gth_file_store_update_visibility (file_store, file_store->priv->queue, position);
 	_gth_file_store_clear_queue (file_store);
 }
 
@@ -1472,6 +1485,8 @@ gth_file_store_queue_set_valist (GthFileStore *file_store,
   			_gth_file_row_set_metadata (row, metadata);
   			row->changed = TRUE;
   			break;
+  		case GTH_FILE_STORE_METADATA_VISIBLE_COLUMN:
+  			break;
   		case GTH_FILE_STORE_CHECKED_COLUMN:
   			row->checked = va_arg (var_args, gboolean);
   			row->changed = TRUE;
@@ -1536,7 +1551,7 @@ gth_file_store_exec_set (GthFileStore *file_store)
 	_gth_file_store_list_changed (file_store);
 	_gth_file_store_clear_queue (file_store);
 	if (file_store->priv->update_filter) {
-		_gth_file_store_update_visibility (file_store, NULL);
+		_gth_file_store_update_visibility (file_store, NULL, -1);
 		file_store->priv->update_filter = FALSE;
 	}
 	else if (file_store->priv->check_changed) {
@@ -1590,7 +1605,7 @@ gth_file_store_exec_remove (GthFileStore *file_store)
 	_gth_file_store_compact_rows (file_store);
 	_gth_file_store_clear_queue (file_store);
 
-	_gth_file_store_update_visibility (file_store, NULL);
+	_gth_file_store_update_visibility (file_store, NULL, -1);
 }
 
 
