@@ -16,8 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef TYPEDEFS_H
@@ -125,6 +124,14 @@ typedef enum {
 } GthOverwriteMode;
 
 
+typedef enum {
+	GTH_GRID_NONE = 0,
+	GTH_GRID_THIRDS,
+	GTH_GRID_GOLDEN,
+	GTH_GRID_CENTER,
+	GTH_GRID_UNIFORM
+} GthGridType;
+
 typedef void (*DataFunc)         (gpointer    user_data);
 typedef void (*ReadyFunc)        (GError     *error,
 			 	  gpointer    user_data);
